/* formulaAddIns.m created by bruehl on Thu 19-Feb-1998 */

#import "formulaAddIns.h"
int search_le(double x, double v[], int n, int tryhere);
static inline double sqr(double a) {return a*a;}
#define IS_NAN(x)	((x)!=(x))	/* IEEE test for NAN (not a number) */

@implementation formulaAddIns

- (id) init:(id) theOwner /* called when the document is opened */
{
    owner = theOwner;
    return [super init];
}


/* this example function returns the sum of its aruments */
- (int) addin0:(double *)result nargs:(int)nargs args:(double *)args
{
    double 	sum=0.0;
    int 	i;

    for (i=0; i<nargs; i++)
        sum += args[i];

    *result = sum;    
    return 0; // return -8 on error
}

/* function call: addin1(curve, x, 1, width) for gaussian filter */
- (int) addin1:(double *)result nargs:(int)nargs args:(double *)args
{
    double 	y, x0;
    int 	i, len, filterType;
	static int iLast;
    double 	*xData, *yData;
    const char	*nameX, *nameY;
    
	if (nargs<4)
		return -8;
	
	[owner curveDataXY:args[0] dataX:&xData dataY:&yData len:&len nameX:&nameX nameY:&nameY];
	if (len<2)
		return -8;
	
	x0 = args[1];
	filterType = args[2];
	
	switch (filterType) {
		case 0: { // test
			*result = 9.6;
			break;
		}
		case 1: { // gaussian filter
			double w, x, sum, sumG, gauss;
			
			w = args[3];
			
			x = x0 - 5*w;
			i = search_le( x, xData, len, iLast);
			x = xData[i];
			iLast = i;
			sum = 0;
			sumG = 0;
			while(i<len) {
				x = xData[i];
				if (x>x0+5*w)
					break;
				y = yData[i];
				gauss = exp(-sqr((x-x0)/w));
				sum += y*gauss;
				sumG += gauss;
				i++;
			}
			
			if (fabs(sumG)<1e-5) {
				*result = 0;
				return 0;
			}
			
			*result = sum/sumG;
			break;
		}
		case 2: { // notch filter
			double freq= args[3];
			double q = args[4];
			double samplerate = len /(xData[len-1]-xData[0]);
			double z1x = cos(2*M_PI*freq/samplerate);
			double a0a2 = (1-q)*(1-q)/(2*(fabs(z1x)+1)) + q;
			double a1 = -2*z1x*a0a2;
			double b1 = -2*z1x*q;
			double b2 = q*q;
			double reg0=0, reg1, reg2;
			
			reg1 = 0;
			reg2 = 0;
			
			for (i = 0; i < len; i++) {
				reg0 = a0a2 * ((double)yData[i] + yData[i+2]) + a1 * yData[i+1] - b1 * reg1 - b2 * reg2;
				
				reg2 = reg1;
				reg1 = reg0;
				if (xData[i]>=x0)
					break;
			}			
			*result = reg0;
			break;
		}
		default:
			return -8;
			break;
	}
    return 0; // return -8 on error
}

- (int) addin2:(double *)result nargs:(int)nargs args:(double *)args
{
    *result = 10.0;
    return 0;
}

- (int) addin3:(double *)result nargs:(int)nargs args:(double *)args
{
    *result = 0.0;
    return 0;
}

- (int) addin4:(double *)result nargs:(int)nargs args:(double *)args
{
    *result = 0.0;
    return 0;
}

- (int) addin5:(double *)result nargs:(int)nargs args:(double *)args
{
    *result = 0.0;
    return 0;
}

- (int) addin6:(double *)result nargs:(int)nargs args:(double *)args
{
    *result = 0.0;
    return 0;
}

- (int) addin7:(double *)result nargs:(int)nargs args:(double *)args
{
    *result = 0.0;
    return 0;
}

- (int) addin8:(double *)result nargs:(int)nargs args:(double *)args
{
    *result = 0.0;
    return 0;
}

- (int) addin9:(double *)result nargs:(int)nargs args:(double *)args
{
    *result = 0.0;
    return 0;
}

@end

int search_le(double x, double v[], int n, int tryhere)
/* return i  v[i]<=x (v must increase) */
{
	int low, high, mid;
	double vm;
	
	low=0;
	high=n-1;
	if (tryhere>=0) {
		double vt=0, vt1=0;  // just to calm compiler
		if (tryhere+1<n) {
			vt = v[tryhere];
			vt1= v[tryhere+1];
			if (vt<=x && x<vt1) return tryhere;
		}
		
		if (tryhere+2<n) {
			if (vt1<=x && x<v[tryhere+2]) return tryhere+1;
		}
		
		if (tryhere+1<n) {
			if (vt<=x) low = tryhere;
			if (x<vt) high = tryhere;
		}
	}
	
	if (IS_NAN(x)) return 0;
	
	while (IS_NAN(v[low]) && low<high) low++;
	if (low==high) return 0;
	if (x <v[low]) return 0;
	if (x==v[low]) return low;
	/* now v[low] is valid */
	
	while (IS_NAN(v[high]) && low<high) high--;
	if (x>=v[high]) return high;
	/* now v[high] is valid */
	
	while(low<high-1) {
		mid = (low+high)/2;
		vm = v[mid];
		if (IS_NAN(vm)) {
			while(IS_NAN(vm) && low<mid) {
				mid--;
				vm = v[mid];} 
			if (low==mid) {
				mid =(low+high)/2;
				vm = v[mid];
				while(IS_NAN(vm) && mid<high) {
					mid++;
					vm = v[mid];}
				if (high==mid) return low;
			}
		}
		/* now vm is valid */
		if (x<vm)
			high = mid;
		else if (x>vm)
			low = mid;
		else
			return(mid);
	}
	return (low);
}